section {
    overflow-x: auto;
    padding-top: 2rem !important;
}

nav, nav > div {
    display: flex;
    align-items: center;
}

a.navbar-item, a.navbar-item:hover {
    color: whitesmoke;
}

a.navbar-item:hover {
    background-color: #292929;
}

div.navbar-end {
    flex-basis: min-content;
}

th.error {
    color: red;
}

img.favicon {
    width: 16px;
    height: 16px;
    position: relative;
    top: 2px;
}

button.fa-button {
    padding: 0;
    cursor: pointer;
    border: none;
}

label.checkbox {
    margin: 1.2rem 1.2rem 0;
}

.hidden {
    display: none;
}

.table td:last-child {
    white-space: nowrap;
}

.table td:last-child form {
    display: inline;
}

.table td.is-primary, .table tr.is-primary {
    background-color: rgba(0, 209, 178, 0.2) !important;
    border-color: rgba(0, 209, 178, 0.1);
    white-space: nowrap;
}

.table td.is-link, .table tr.is-link {
    background-color: rgba(50, 115, 220, 0.2) !important;
    border-color: rgba(50, 115, 220, 0.1);
    white-space: nowrap;
}

.table td.is-info, .table tr.is-info {
    background-color: rgba(32, 156, 238, 0.2) !important;
    border-color: rgba(32, 156, 238, 0.1);
    white-space: nowrap;
}

.table td.is-success, .table tr.is-success {
    background-color: rgba(35, 209, 96, 0.2) !important;
    border-color: rgba(35, 209, 96, 0.1);
    white-space: nowrap;
}

.table td.is-warning, .table tr.is-warning {
    background-color: rgba(255, 221, 87, 0.2) !important;
    border-color: rgba(255, 221, 87, 0.1);
    white-space: nowrap;
}

.table td.is-danger, .table tr.is-danger {
    background-color: rgba(255, 56, 96, 0.2) !important;
    border-color: rgba(255, 56, 96, 0.1);
    white-space: nowrap;
}

#quick_scroll {
    margin-bottom: 1rem;
}

.table.is-hoverable tbody tr:not(.is-selected):hover {
    background-color: #f1f1f1 !important;
}
